from Products.Archetypes.public import *
from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.ExternalStorage.ExternalStorage import ExternalStorage
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.debug import log
from StreamingFile import StreamingFile

# check to see if you have the supplementary validators installed 
try:
    from Products.validation.validators.SupplValidators import MaxSizeValidator
    HAS_MAXSIZE_VALIDATOR = True
except ImportError:
    HAS_MAXSIZE_VALIDATOR = False
    pass

# check to see if there is a user-defined value for max_file_size
#portal = context.portal_url.getPortalObject()
#max_file_size = getattr(getToolByName(portal, 'portal_properties').ataudio_properties, 'max_file_size', None)

# otherwise, use the default defined in config.py
#if max_file_size == 'None':
#    max_file_size = MAX_FILE_SIZE

from OFS.Image import File
from config import *
from cStringIO import StringIO

from ID3Storage import ID3Storage
from Products.Archetypes.Registry import registerStorage

def initialize(context):
    registerStorage(ID3Storage)

def cleanupFilename(filename):
    result = ''
    for s in str(filename):
        s = CHAR_MAPPING.get(s, s)
        if s in GOOD_CHARS:
            result+=s
    return result

schema = BaseSchema +  Schema((
    StringField('id',
                searchable=0,
                required=1,
                visible=0,
                widget=StringWidget(label='id', visible = {'edit':'hidden', 'view':'visible'}   ),
                ),
                            
    StringField('description',
                searchable=1,
                storage=MetadataStorage(),
                accessor="Description",
                widget=TextAreaWidget(label="Description",
                                    label_msgid="label_audiofile_description",
                                    description="Enter a short description of this audio file.",
                                    description_msgid="desc_audiofile_description",
                                    i18n_domain="ataudio")
                ),

    FileField('file',
                primary=1,
                required=1,
                validators=MaxSizeValidator("checkFileMaxSize", maxsize=MAX_FILE_SIZE),
                storage=ExternalStorage(prefix="files", archive=False),
                widget=FileWidget(label="Audio File",
                                    label_msgid="label_audiofile_file",
                                    description="Upload your MP3 file and if it has ID3 tags, the fields on this form will be auto-populated.",
                                    description_msgid="desc_audiofile_file",
                                    i18n_domain="ataudio")
                ),

    StringField('title',
                searchable=1,
                accessor="Title",
                mutator="setTitle",
                storage=ID3Storage(),
                widget=StringWidget(label="Title", 
                                    label_msgid="label_audiofile_title",
                                    description="The title of this audio file.",
                                    description_msgid="desc_audiofile_title",
                                    i18n_domain="ataudio"), 
                ),
                
    StringField('artist',
                searchable=1,
                accessor="Artist",
                storage=ID3Storage(),
                widget=StringWidget(label="Artist", 
                                    label_msgid="label_audiofile_artist",
                                    description="The title of this audio file.",
                                    description_msgid="desc_audiofile_artist",
                                    i18n_domain="ataudio"), 
                ),

    StringField('album',
                searchable=1,
                accessor="Album",
                storage=ID3Storage(id3tag="album"),
                widget=StringWidget(label="Album", 
                                    label_msgid="label_audiofile_album",
                                    description="The album that this audio file is on",
                                    description_msgid="desc_audiofile_album",
                                    i18n_domain="ataudio"), 
                ),

    StringField('year',
                searchable=1,
                accessor="Year",
                storage=ID3Storage(),
                default='',
                widget=IntegerWidget(label="Year",
                                     label_msgid="label_audiofile_year",
                                     description="Enter the year that this audio file was recorded.",
                                     description_msgid="desc_audiofile_year",
                                     i18n_domain="ataudio", 
                                     size="4",
                                     ),
                ),
                
# iTunes does not seem to store the length of the song, so this breaks the accessor
#    StringField('songlen',
#                storage=ID3Storage(),
#                accessor="songlen",
#                widget=IntegerWidget(label='Song length',
#                                     description='Enter the length of the song in seconds.',
#                                     size='4'
#                                    )
#                ),
              
# until we know how to better handle multiple genres, comment this code out               
#    StringField('genre',
#                vocabulary=GENRES,
#                widget=SelectionWidget(description='Select one of the following genres. more to come!'),
#                ),

    ),
    marshall=PrimaryFieldMarshaller(),
   )

class ATAudio(BaseContent):
    """This is the ATAudio class which has a view action and file method.
    """

    schema = schema

    content_icon   = 'audio_icon.gif'
    meta_type      = 'ATAudio'
    archetype_name = 'Audio file'
    immediate_view = 'audio_view'
    default_view   = 'audio_view'
    suppl_views    = ()
    typeDescription= "Add the relevant details of the audio file to be added in the form below,\n" \
                     "select the audio file with the 'Browse' button, and press 'Save'."
    typeDescMsgId  = "description_edit_audio"
    assocMimetypes = ('audio/mpeg' )
    assocFileExt   = ('mp3')
    
    actions = (
       {'id'            : 'view',
        'name'          : 'View',
        'action'        : 'string:${object_url}/audio_view',
        'permissions'   : (CMFCorePermissions.View,)
        },
       {'id'            : 'stream',
        'name'          : 'Stream',
        'action'        : 'string:${object_url}/stream',
        'permissions'   : (CMFCorePermissions.View, )
         },
       {'id'            : 'download',
        'name'          : 'Download',
        'action'        : 'string:${object_url}/file',
        'permissions'   : (CMFCorePermissions.View, )
         },
       )
        
# not sure if these are necessary

    def setDescription(self, value):
        """set the description"""
        field = self.getField('description')
        field.set(self, value)

    def getDescription(self):
        """get the description"""
        field = self.getField('description')
        return field.getvalue()
        
    def setTitle(self, value):
        """simple method to set the title"""
        field = self.getField('title')
        field.set(self, value.encode('utf-8'))

    def setArtist(self, value):
        """simple method to set the artist"""
        field = self.getField('artist')
        field.set(self, value)

    def setAlbum(self, value):
        """simple method to set the album"""
        field = self.getField('album')
        field.set(self, value)

    def setYear(self, value):
        """simple method to set the year"""
        field = self.getField('year')
        field.set(self, value)
        
    def setFile(self,value,field="file", **kw):
        """example mutator which does rename the object when uploading - thanks to MrTopf & Tiran

        you can omit this mutator if you don't want the object to be named the same
        as the uploaded file. But it makes things usually easier and this might even
        go into the actual storage to have things together.
        
        """
        f=self.Schema()['file']
        # get filename from uploaded data
        filename=getattr(value,'filename',None)
        if filename:
            # M$ hack
            filename=filename.split("\\")[-1]
            # is it different than we're called right now
            if filename!=self.getId():
                # cleanup filename
                clean_filename = cleanupFilename(filename)
                # call plone rename
                self.plone_utils.contentEdit(self,id=clean_filename)

        # call fields setter
        return f.set(self, value, **kw)
    

    def getAudioURL(self, media_server):
        """get the URL for the audio file"""
        relative_url='/'.join(getToolByName(self, 'portal_url').getRelativeContentPath(self))
        out = StringIO()
        out.write('http://%s/%s' % (media_server, relative_url))
        return out.getvalue()
                   
    def file(self,**kwargs):
        """downloads the audio file with proper content type"""
 	# get the file's content type
	ct = self.getContentType('file')
        REQUEST=kwargs.get('REQUEST',self.REQUEST)
        RESPONSE=kwargs.get('RESPONSE',REQUEST.RESPONSE)
        RESPONSE.setHeader('Content-Type', ct)  #the content type should be taken from the FileField like the field.pt widget do
        RESPONSE.setHeader('Content-Disposition', 'attachment; filename="%s"' % self.getId()) #the file ids are supposed to be *.mp3/avi etc.

#        file = self.getFile()
        file = self.getFile(useIterator=1)
        path = self.Schema()['file'].storage.getFilepath(self,'file')
        # what about size?!?
#        return File(self.getId(),self.Title(),file,ct).__of__(self.aq_parent).index_html(REQUEST,RESPONSE)
        return StreamingFile(self.getId(),self.Title(),file,ct).__of__(self.aq_parent).index_html(REQUEST,RESPONSE,path)

    def stream(self, REQUEST, RESPONSE):
        """stream the audio file as M3U"""
        RESPONSE.setHeader('Content-Type', 'audio/x-mpegurl')
        RESPONSE.setHeader('Content-Disposition', 'attachment; filename="%s.m3u"' % self.getId())

        media_server = getattr(getToolByName(self, 'portal_properties').ataudio_properties, 'media_server', None)

        if media_server:
            audiofile_url = self.getAudioURL(media_server)
        else:
            # if no media_server has been defined in portal_properties -> ataudio_properties, 
            # use Zope as the media server, which is more efficient now thanks to the filestream iterator
            audiofile_url = REQUEST.URL1 + '/file'
            
        out = StringIO()
        # in order to conform to the standard for M3U files (http://hanna.pyxidis.org/tech/m3u.html)
#        out.write('#EXTM3U\n')
        # the songlen accessor is broken, so we hardcode 120 seconds as the song length for now
#        out.write('#EXTINF:%s,%s\n' % (self.songlen(), self.Title()))
#        out.write('#EXTINF:%s,%s\n' % (120, self.Title()))
        out.write(audiofile_url + '\n')
        return out.getvalue()
        
    # make streaming the default method when entering the objects URL
#    index_html=file
    
registerType(ATAudio, PROJECTNAME)
