from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName


""" 
   ATAudioFolder gives you a track listing of your audio files and an optional image
"""

schema = BaseFolderSchema +  Schema((
    StringField('title',
                required=1,
                searchable=1,
                default='',
                accessor='Title',
                widget=StringWidget(label="CD Title",
                                    label_msgid="label_cd_title",
                                    description="The title for this CD",
                                    description_msgid="desc_cd_title",
                                    i18n_domain="ataudio"),
                ),    
    StringField('description',
                required=1,
                storage=MetadataStorage(),
                searchable=1,
                accessor='Description',
                widget=TextAreaWidget(label='Description',
                                    label_msgid="label_cd_description",
                                    description='Give a description for this CD.',
                                    description_msgid="desc_cd_description",
                                    i18n_domain="ataudio"), 
               ),
    ImageField('coverimage',
               required = 0,
               original_size = (600,600),
               sizes= {'large':(256,256),
                       'medium':(128,128),
                       'small':(64,64)
                      },
               widget = ImageWidget(label= "CD cover image",
                                    label_msgid = "label_cd_coverimage",
                                    description = "Upload a CD cover image. It will be automatically resized",
                                    description_msgid = "desc_cd_coverimage",
                                    i18n_domain = "ataudio")
               ),

    StringField('artist',
                searchable=1,
                widget=StringWidget(label='Artist', 
                                    label_msgid="label_cd_artist",
                                    description='The artists name.',
                                    description_msgid="desc_cd_artist",
                                    i18n_domain="ataudio"), 
               ),
    
    StringField('label',
                searchable=1,
                widget=StringWidget(label='Record Label', 
                                    label_msgid="label_cd_recordlabel",
                                    description='Label this CD was issued on.',
                                    description_msgid="desc_cd_recordlabel",
                                    i18n_domain="ataudio"), 
               ),

    IntegerField('year',
                searchable=1,
                default='',
                validators=('isInt',),
                widget=IntegerWidget(label='Year', 
                                    label_msgid="label_cd_year",
                                    description='Year this CD was issued (i.e. 1999).',
                                    description_msgid="desc_cd_year",
                                    size="4",
                                    i18n_domain="ataudio"), 
               ),

# experimental feature for selling CDs via amazon.com
#    StringField('itemnum',
#                accessor="getItemnum",
#                widget=StringWidget(label='ASIN',
#                                    description='Amazon ID (ASIN number) of this item',
#                                    size=12,),
#               ),
    ))
        
class ATAudioFolder(OrderedBaseFolder):
    """
        A folder for containing audio files, i.e. a CD
    """
    schema = schema
    filter_content_types = 1
    allowed_content_types = ('ATAudio','File') 
    meta_type = 'ATAudioFolder'
    archetype_name = 'CD'
    
    content_icon = 'cdaudio_icon.png'
    actions = ({
       'id': 'view',
        'name': 'View',
        'action': 'string:${object_url}/audiofolder_view',
        'permissions': (CMFCorePermissions.View,)
        },
        {'id': 'references',
          'name': 'References',
          'action': 'string:${object_url}/reference_edit',
          'permissions': (CMFCorePermissions.ModifyPortalContent,),
          'visible':0},
        {'id': 'metadata',
          'name': 'Properties',
          'action': 'string:${object_url}/base_metadata',
          'permissions': (CMFCorePermissions.ModifyPortalContent,),
          'visible':0})    

    def getAllowedTypes(self):
        """
        Let's plumb that content listing. Strip out 'File' and only show 'Audio file' in the add new item dropdown.
        """
#        types = self.sortObjects( self.allowedContentTypes() )
#        return [ ctype for ctype in types if ctype.getId() in ('ATAudio',) ]               

        pt=getToolByName(self,'portal_types')
        ataudio=getattr(pt,'ATAudio')
        return [ataudio]

#     def getItemnum(self):
#         asin = getattr(self, 'itemnum', None)
#         if not asin:
#             return asin
#         return asin.strip()
        
registerType(ATAudioFolder)
