from Products.ExternalStorage.interfaces import IExternalStorage
from Products.Archetypes.interfaces.storage import IStorage
from Products.Archetypes.interfaces.layer import ILayer
from Products.Archetypes.debug import log
from Products.Archetypes.Storage import StorageLayer, type_map

from Acquisition import aq_base

try:
    import pyid3lib
except ImportError:
    log("You are missing pyid3lib which means ATAudio won't be able to extract ID3 tags from your MP3 files. Please see docs/ID3.txt for instructions on how to install pyid3lib.")
    HAS_PYID3LIB = False

import os

CACHE_PREFIX="_v_id3_"

class ID3Storage(StorageLayer):
    __implements__ = (IStorage, ILayer)

    def __init__(self, id3tag=None):
        self.id3tag = id3tag

    def getTag(self, instance):
        "Return the tag object for an instance"
        instance = aq_base(instance)
        path = instance.getField('file').storage.getFilepath(instance,'file')

        #log("ID3Storage: getting cached tag for: %s" % path)
        if not os.path.isfile(path):
            log("ID3Storage: error, trying to get tag of non-file: %s" % path)
            raise AttributeError("MP3 file not created, no ID3 tag available")

        return pyid3lib.tag(path)

    def get(self, name, instance, **kwargs):
        name = self.id3tag or name
        #log("ID3Storage: (%s) get %s " % (repr(instance), repr(name)))

        try: 
            value = getattr(self.getTag(instance), name)
            #log("ID3Storage: got attribute %s: %s" % (name,value))
            return value
        except AttributeError, e: 
            log("ID3Storage: failed to get attribute %s (%s)" % (name, str(e)))
            return None

    def set(self, name, instance, value, **kwargs):
        name = self.id3tag or name
        instance = aq_base(instance)
        value = aq_base(value)
        try:
            #log("ID3Storage: (%s) set %s (%s): " % (repr(instance), repr(name), repr(value)))
            tag = self.getTag(instance)
            setattr(tag, self.id3tag or name, str(value.encode('iso8859')))
            tag.update()
            #log("ID3Storage: setted (%s): %s" % (repr(name), getattr(tag, name)))
        except AttributeError, e:
            log("ID3Storage: failed to set attribute %s (%s)" % (name, str(e)))
            pass

    def unset(self, name, instance, **kwargs):
        log("ID3Storage: unset %s: " % repr(name))
        try: setattr(self.getTag(instance), self.id3tag or name, '')
        except AttributeError: pass

    def initializeInstance(self, instance, item=None, container=None):
        log("ID3Storage: initializeInstance: " + repr(instance))
#if IExternalStorage in instance.Schema()['file'].storage.__implements__:
#setattr(aq_base(instance), '_v_tag', tag(instance.getField('file').storage.getFilepath(instance)))

    def cleanupInstance(self, instance, item=None, container=None):
        log("ID3Storage: cleanupInstance: " + repr(instance))
#if IExternalStorage in instance.Schema()['file'].storage.__implements__:
#delattr(aq_base(instance), '_v_tag')

    def initializeField(self, instance, field):
        log("ID3Storage: initializeField: " + repr(field))

    def cleanupField(self, instance, field):
        log("ID3Storage: cleanupField: " + repr(field))
