from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions

""" 
   Recordings is a container for ATAudioFolder objects. For example, you could use
   it to show a list of all of your CDs.
"""

schema = BaseFolderSchema +  Schema((
    StringField('title',
                required=1,
                searchable=1,
                default='',
                accessor='Title',
                widget=StringWidget(label="Title",
                                    label_msgid="label_cd_title",
                                    description="The title for this list of recordings",
                                    description_msgid="desc_recordings_title",
                                    i18n_domain="ataudio"),
                ),    
    StringField('description',
                required=1,
                storage=MetadataStorage(),
                searchable=1,
                accessor='Description',
                widget=TextAreaWidget(label='Description',
                                    label_msgid="label_recordings_description",
                                    description='Description for this list of recordings.',
                                    description_msgid="desc_recordings_description",
                                    i18n_domain="ataudio"), 
               ),
    ))
        
class Recordings(OrderedBaseFolder):
    """
        A recordings folder is a container for your CDs.
    """
    schema = schema
    filter_content_types = 1
    allowed_content_types = 'ATAudioFolder',
    meta_type = 'Recordings'
    archetype_name = 'Recordings'
    
    content_icon = 'folder_icon.gif'
    actions = ({
       'id': 'view',
        'name': 'View',
        'action': 'string:${object_url}/recordings_view',
        'permissions': (CMFCorePermissions.View,)
        },
        {'id': 'references',
          'name': 'References',
          'action': 'string:${object_url}/reference_edit',
          'permissions': (CMFCorePermissions.ModifyPortalContent,),
          'visible':0},
        {'id': 'metadata',
          'name': 'Properties',
          'action': 'string:${object_url}/base_metadata',
          'permissions': (CMFCorePermissions.ModifyPortalContent,),
          'visible':0})    

registerType(Recordings)
